"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.signRequest = signRequest;
exports.processApiResult = processApiResult;
exports.paginateApi = paginateApi;
exports.findIds = findIds;

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const axios = require(`axios`);

const crypto = require('crypto');

const _require = require('./constants'),
      marvelAPI = _require.marvelAPI;

function signRequest({
  publicKey,
  privateKey
}) {
  const now = Date.now();
  return {
    hash: crypto.createHash('md5').update(`${now}${privateKey}${publicKey}`).digest("hex"),
    ts: now,
    apikey: publicKey
  };
}

function processApiResult(result) {
  // TODO verify response status, etag etc
  return result.data.data;
}

function paginateApi(_x) {
  return _paginateApi.apply(this, arguments);
}

function _paginateApi() {
  _paginateApi = _asyncToGenerator(function* (_ref) {
    let route = _ref.route,
        limit = _ref.limit,
        params = _ref.params,
        options = _objectWithoutProperties(_ref, ["route", "limit", "params"]);

    let offset = 0;
    let total = 1;
    let page = 1;
    let results = [];
    let result, processed;

    while (results.length < total) {
      result = yield axios.get(route, {
        params: _objectSpread({
          limit,
          offset
        }, params, signRequest(options))
      });
      page = page + 1;
      offset = limit * (page - 1);
      processed = processApiResult(result);
      results = results.concat(processed.results);
      total = processed.total;
    }

    return results;
  });
  return _paginateApi.apply(this, arguments);
}

function findIds(_x2) {
  return _findIds.apply(this, arguments);
}

function _findIds() {
  _findIds = _asyncToGenerator(function* (_ref2) {
    let entity = _ref2.entity,
        entityFilter = _ref2.entityFilter,
        options = _objectWithoutProperties(_ref2, ["entity", "entityFilter"]);

    const results = yield paginateApi(_objectSpread({
      route: `${marvelAPI}${entity}`,
      params: _objectSpread({}, entityFilter)
    }, options));
    return results.map(result => result.id);
  });
  return _findIds.apply(this, arguments);
}